@@ -1,7 +1,7 @@
 package App::resolvetable;
 
-our $DATE = '2019-05-28'; # DATE
-our $VERSION = '0.002'; # VERSION
+our $DATE = '2019-05-29'; # DATE
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
@@ -9,6 +9,7 @@ use warnings;
 use Log::ger;
 
 use Color::ANSI::Util qw(ansifg);
+use Time::HiRes qw(time);
 
 our %SPEC;
 
@@ -55,11 +56,34 @@ sub _colorize_maj_min {
     }
 }
 
+sub _mark_undef_with_x {
+    my $row = shift;
+    for (keys %$row) {
+        next if $_ eq 'name';
+        next if defined $row->{$_};
+        $row->{$_} = ansifg("ff0000")."X"."\e[0m";
+    }
+}
+
 $SPEC{'resolvetable'} = {
     v => 1.1,
     summary => 'Produce a colored table containing DNS resolve results of '.
         'several names from several servers',
     args => {
+        action => {
+            schema => ['str*', in=>[qw/show-addresses show-times/]],
+            default => 'show-addresses',
+            cmdline_aliases => {
+                times => {is_flag=>1, summary=>'Shortcut for --action=show-times', code=>sub { $_[0]{action} = 'show-times' }},
+            },
+            description => <<'_',
+
+The default action is to show resolve result (`show-addresses`). If set to
+`show-times`, will show resolve times instead to compare speed among DNS
+servers.
+
+_
+        },
         servers => {
             'x.name.is_plural' => 1,
             'x.name.singular' => 'server',
@@ -102,16 +126,21 @@ sub resolvetable {
     my $type = $args{type} // "A";
     my $names   = $args{names};
     my $servers = $args{servers};
+    my $action  = $args{action} // 'show-addresses';
 
-    my %res; # key=name, val={server=>result, ...}
+    my %res;       # key=name, val={server=>result, ...}
+    my %starttime; # key=name, val={server=>time, ...}
+    my %endtime  ; # key=name, val={server=>time, ...}
 
     log_info "Resolving ...";
     my $resolver = Net::DNS::Async->new(QueueSize => 30, Retries => 2);
     for my $name (@$names) {
         for my $server (@$servers) {
+            $starttime{$name}{$server} = time();
             $resolver->add({
                 Nameservers => [$server],
                 Callback    => sub {
+                    my $time = time();
                     my $pkt = shift;
                     return unless defined $pkt;
                     my @rr = $pkt->answer;
@@ -122,6 +151,7 @@ sub resolvetable {
                             (length($res{ $k }{$server}) ? ", ":"") .
                             $r->address
                             if $r->type eq $type;
+                        $endtime{$name}{$server} //= $time;
                     }
                 },
             }, $name, $args{type});
@@ -132,11 +162,41 @@ sub resolvetable {
     log_trace "Returning table result ...";
     my @rows;
     for my $name (@$names) {
-        my $row = {
-            name => $name,
-            map { $_ => $res{$name}{$_} } @$servers,
-        };
-        _colorize_maj_min($row) if $args{colorize};
+        my $row;
+        if ($action eq 'show-addresses') {
+            $row = {
+                name => $name,
+                map { $_ => $res{$name}{$_} } @$servers,
+            };
+            _colorize_maj_min($row) if $args{colorize};
+            _mark_undef_with_x($row) if $args{colorize};
+        } elsif ($action eq 'show-times') {
+            $row = {
+                name => $name,
+                map {
+                    my $server = $_;
+                    my $starttime = $starttime{$name}{$_};
+                    my $endtime   = $endtime{$name}{$_};
+                    my $val;
+                    if (defined $endtime) {
+                        my $ms = ($endtime - $starttime)*1000;
+                        if ($ms > 4000) {
+                            $val = ">4000ms";
+                        } elsif ($ms < 1) {
+                            $val = "   <1ms";
+                        } else {
+                            $val = sprintf("%3.0fms", $ms);
+                        }
+                    } else {
+                        $val = undef;
+                    }
+                    ($server => $val);
+                } @$servers,
+            };
+        } else {
+            die "Unknown action '$action'";
+        }
+        _mark_undef_with_x($row) if $args{colorize};
         push @rows, $row;
     }
 
@@ -158,7 +218,7 @@ App::resolvetable - Produce a colored table containing DNS resolve results of se
 
 =head1 VERSION
 
-This document describes version 0.002 of App::resolvetable (from Perl distribution App-resolvetable), released on 2019-05-28.
+This document describes version 0.003 of App::resolvetable (from Perl distribution App-resolvetable), released on 2019-05-29.
 
 =head1 FUNCTIONS
 
@@ -177,6 +237,12 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<action> => I<str> (default: "show-addresses")
+
+The default action is to show resolve result (C<show-addresses>). If set to
+C<show-times>, will show resolve times instead to compare speed among DNS
+servers.
+
 =item * B<colorize> => I<bool>
 
 =item * B<names>* => I<array[str]>
